package IO::K8s::Role::HelmManaged;
# ABSTRACT: Role for K3s Helm chart management
our $VERSION = '1.008';
use Moo::Role;

sub from_repo {
    my ($self, $repo_url, $chart_name) = @_;
    my $spec = $self->spec // {};
    $spec->{repo}  = $repo_url;
    $spec->{chart} = $chart_name;
    $self->spec($spec);
    return $self;
}

sub set_version {
    my ($self, $version) = @_;
    my $spec = $self->spec // {};
    $spec->{version} = $version;
    $self->spec($spec);
    return $self;
}

sub set_values {
    my ($self, %values) = @_;
    my $spec = $self->spec // {};
    my $existing = $spec->{set} // {};
    @{$existing}{keys %values} = values %values;
    $spec->{set} = $existing;
    $self->spec($spec);
    return $self;
}

sub set_values_yaml {
    my ($self, $yaml_str) = @_;
    my $spec = $self->spec // {};
    $spec->{valuesContent} = $yaml_str;
    $self->spec($spec);
    return $self;
}

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

IO::K8s::Role::HelmManaged - Role for K3s Helm chart management

=head1 VERSION

version 1.008

=head1 SUPPORT

=head2 Issues

Please report bugs and feature requests on GitHub at
L<https://github.com/pplu/io-k8s-p5/issues>.

=head2 IRC

Join C<#kubernetes> on C<irc.perl.org> or message Getty directly.

=head1 CONTRIBUTING

Contributions are welcome! Please fork the repository and submit a pull request.

=head1 AUTHORS

=over 4

=item *

Torsten Raudssus <torsten@raudssus.de>

=item *

Jose Luis Martinez <jlmartin@cpan.org> (original author, inactive)

=back

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2018 by Jose Luis Martinez.

This is free software, licensed under:

  The Apache License, Version 2.0, January 2004

=cut
